import { system, world } from '@minecraft/server';
import { decrementStack, getOppositeDirection, DirectionType, cardinalSides } from './utils/helper';
import { directionToVector3 } from './utils/math';

world.beforeEvents.worldInitialize.subscribe(initEvent => {
  initEvent.blockComponentRegistry.registerCustomComponent('jsb_cucawall:trigger', {
  onPlayerDestroy: e => { for (const direction of DirectionType.HORIZONTAL) {
    const directionOpposite = getOppositeDirection(direction);
    const blockToCheck = e.block.offset(directionToVector3(directionOpposite));

    for (const directionChild of DirectionType.HORIZONTAL) {
        const directionOppositeChild = getOppositeDirection(directionChild);
        const blockToCheckChild = blockToCheck.offset(directionToVector3(directionOppositeChild));

        try {
            blockToCheck.setPermutation(blockToCheck.permutation.withState(`pa:${directionOppositeChild.toLowerCase()}`, !blockToCheckChild.typeId.includes("minecraft:air")));
        } catch (error) { }
    }
}
 },
beforeOnPlayerPlace: e => { e.permutationToPlace = e.permutationToPlace.withState("pa:in_world", true);

for (const direction of DirectionType.HORIZONTAL) {
    const directionOpposite = getOppositeDirection(direction);
    const blockToCheck = e.block.offset(directionToVector3(directionOpposite));

    if (blockToCheck.typeId.includes("minecraft:air")) continue;
    e.permutationToPlace = e.permutationToPlace.withState(`pa:${directionOpposite.toLowerCase()}`, !blockToCheck.typeId.includes("minecraft:air"));

    try {
        blockToCheck.setPermutation(blockToCheck.permutation.withState(`pa:${direction.toLowerCase()}`, true));
    } catch (error) { }
}
 },
});

initEvent.blockComponentRegistry.registerCustomComponent('jsb_cucafence:trigger', {
  onPlayerInteract: e => { let face = getOppositeDirection(e.face);

const isOpen = e.block.permutation.getState("pa:open");
const sound = isOpen ? "mud_bricks" : "mud_bricks";

e.block.setPermutation(e.block.permutation.withState("minecraft:cardinal_direction", face.toLowerCase()));
e.block.setPermutation(e.block.permutation.withState("pa:open", !isOpen));

e.dimension.playSound(sound, e.block.center(), {
    pitch: 0.9,
    volume: 0.9,
});
 },
});

initEvent.blockComponentRegistry.registerCustomComponent('jsb_guaranasapling:trigger', {
  onRandomTick: e => { const growthChance = 1 / 3;
if (Math.random() > growthChance) return;

const growth = e.block.permutation.getState("pa:growth");
e.block.setPermutation(
    e.block.permutation.withState("pa:growth", growth + 1)
);
 },
onPlayerInteract: e => { if (!e.player) return;

const equippable = e.player.getComponent("minecraft:equippable");
if (!equippable) return;

const mainhand = equippable.getEquipmentSlot("Mainhand");
if (!mainhand.hasItem() || mainhand.typeId !== "minecraft:bone_meal")
    return;

let growth = e.block.permutation.getState("pa:growth");

if (growth >= 6) return;

growth += 1;
e.block.setPermutation(
    e.block.permutation.withState("pa:growth", growth)
);

if (mainhand.amount > 1) mainhand.amount--;
else mainhand.setItem(undefined);

const effectLocation = e.block.center();
e.dimension.playSound("item.bone_meal.use", effectLocation);
e.dimension.spawnParticle(
    "minecraft:crop_growth_emitter",
    effectLocation
);
 },
});

});
